defmodule Dinge.Ledger do
  @moduledoc """
  The Ledger context.
  """

  import Ecto.Query, warn: false
  alias Dinge.Repo

  alias Dinge.Ledger.Ding

  @doc """
  Returns the list of dings.

  ## Examples

      iex> list_dings()
      [%Ding{}, ...]

  """
  def list_dings do
    Repo.all(Ding)
  end

  def list_dings_by_user(user) do
    query =
      from d in Ding,
      where: d.user_id == ^user.id
    Repo.all(query)
  end

  @doc """
  Gets a single ding.

  Raises `Ecto.NoResultsError` if the Ding does not exist.

  ## Examples

      iex> get_ding!(123)
      %Ding{}

      iex> get_ding!(456)
      ** (Ecto.NoResultsError)

  """
  def get_ding!(id), do: Repo.get!(Ding, id)

  @doc """
  Creates a ding.

  ## Examples

      iex> create_ding(%{field: value})
      {:ok, %Ding{}}

      iex> create_ding(%{field: bad_value})
      {:error, %Ecto.Changeset{}}

  """
  def create_ding(attrs \\ %{}) do
    %Ding{}
    |> Ding.changeset(attrs)
    |> Repo.insert()
  end

  @doc """
  Updates a ding.

  ## Examples

      iex> update_ding(ding, %{field: new_value})
      {:ok, %Ding{}}

      iex> update_ding(ding, %{field: bad_value})
      {:error, %Ecto.Changeset{}}

  """
  def update_ding(%Ding{} = ding, attrs) do
    ding
    |> Ding.changeset(attrs)
    |> Repo.update()
  end

  @doc """
  Deletes a ding.

  ## Examples

      iex> delete_ding(ding)
      {:ok, %Ding{}}

      iex> delete_ding(ding)
      {:error, %Ecto.Changeset{}}

  """
  def delete_ding(%Ding{} = ding) do
    Repo.delete(ding)
  end

  @doc """
  Returns an `%Ecto.Changeset{}` for tracking ding changes.

  ## Examples

      iex> change_ding(ding)
      %Ecto.Changeset{data: %Ding{}}

  """
  def change_ding(%Ding{} = ding, attrs \\ %{}) do
    Ding.changeset(ding, attrs)
  end
end
